package Logoved::Role::Stream::Out::Listener::Repocop::Polytest;
use strict;
use warnings;
use Moo::Role;
with 'Logoved::Role::Stream::Out::Listener::Repocop';

has unhandled_messages => (is => 'ro', default => sub { [] });
has tests => (is => 'lazy');
requires 'nested_test_classes';

sub _build_tests {
    my ($self)=@_;
    local $_;
    my $listeners_factory=$self->listeners_factory;
    my $stream=$self->stream;
    my @tests;
    foreach my $class ($self->nested_test_classes) {
	push @tests, $class->new( stream => $stream, $listeners_factory->get_listener_args($class) );
    }
    return \@tests;
}

sub subscribe_stream {
    my ($class, $stream)=@_;
    my ($nested_class)=$class->nested_test_classes;
    return $nested_class->subscribe_stream($stream);
}


sub write {
    my ($self, $line)=@_;
    my $tests=$self->tests;
    foreach my $test (@$tests) {
	return 1 if $test->write($line);
    }
    $self->check_unhandled_messages($line);
}

sub check_unhandled_messages {
    my ($self, $line)=@_;
    if ($line =~ /(?:^|: )(?:error|warning|ERROR|WARNING): /) {
	warn "Repocop::Polytest: unhandled line: ", $line;
	push @{$self->unhandled_messages}, $line;
    }
}

sub commit {
    my $self=shift;
    my $tests=$self->tests;
    foreach my $test (@$tests) {
	$test->commit;
    }
    # TODO unhandled_messages
}

1;

__END__
sub commit {
    my $self=shift;
    my $tests=$self->tests;
    foreach my $barename (keys(%$tests)) {
	$tests->{'barename'}->commit;
    }
}
